local GH = GarrisonHelper
local L = GH.L
-- Frame and Events
local GarrisonHelper_Events = CreateFrame("Frame");
local events = {};

--[[
Name		Ore		Dust	Fur		Herbs	Hide	Meat	Flesh
Yula		4*		5		4*		8		10		5		8
Talgaiir 	10		10		5		5		4*		8		4*
Portanuus	6		6		6		6		6		6		6

Krixel		6		8		10		10		8		4*		5
Pyxni		8		4*		8		4*		8		10		10

Surehide	4*		5		4*		8		10		5		8
Ribchewer	10		10		5		5		4*		8		4*
Tai'tasi	6		6		6		6		6		6		6
]]--

GH.npcMessages = {
	[86779] = {"Krixel", L["Meat"], {109131,109132,109133,109134,109135,109136,}},
	[87200] = {"Krixel", L["Meat"], {109131,109132,109133,109134,109135,109136,}},
	[86778] = {"Pyxni", L["Dust & Herbs"], {109693,109124,109125,109126,109127,109128,109129,}},
	[87201] = {"Pyxni", L["Dust & Herbs"], {109693,109124,109125,109126,109127,109128,109129,}},
	[86776] = {"Ribchewer", L["Leather & Fish"],{110609,109137,109138,109139,109140,109141,109142,109143,109144,}},
	[87203] = {"Talgaiir", L["Leather & Fish"],{110609,109137,109138,109139,109140,109141,109142,109143,109144,}},
	[86777] = {"Surehide", L["Ore & Cloth"],{109118,109119,111557,}},
	[87202] = {"Yula", L["Ore & Cloth"],{109118,109119,111557,}},
	[86683] = {"Tai'tasi", "-", {}},
	[87204] = {"Portanuus", "-", {}},
}

GH.townhallNPCs = {
	--http://www.wowhead.com/quests?filter=ty=81;cr=8;crs=1;crv=60003
	[89763] = {"Muradin Bronzebeard", {{{175, 114677},{616000,92601}},{{1, 123975},{616000,92601}}}, L["Dungeon Quest or Raid Quests"]},
	[91196] = {"Muradin Bronzebeard", {{{175, 114677},{616000,92601}},{{1, 123975},{616000,92601}}}, L["Dungeon Quest or Raid Quests"]},
	[89753] = {"High Overlord Saurfang", {{{175, 114677},{616000,92601}},{{1, 123975},{616000,92601}}}, L["Dungeon Quest or Raid Quests"]},
	[91195] = {"High Overlord Saurfang", {{{175, 114677},{616000,92601}},{{1, 123975},{616000,92601}}}, L["Dungeon Quest or Raid Quests"]},
	--http://www.wowhead.com/quests?filter=na=death;cr=27:8;crs=1:1;crv=0:60003
	[89805] = {"Renzik 'The Shiv'", {{{816200,92601}}}, L["Garrison Support"]},
	[89806] = {"Shadow Hunter Ty'jin", {{{816200,92601}}}, L["Garrison Support"]},
	--profession
	[91024] = {"Jake the Fox", {110609}, L["Profession Quest"]},
	[91033] = {"Zeezu", {110609}, L["Profession Quest"]},
	[91025] = {"Dorothy 'Two'", {111557}, L["Profession Quest"]},
	[91034] = {"Calvo Klyne", {111557}, L["Profession Quest"]},
	[91020] = {"Enchantress Ismae", {109693}, L["Profession Quest"]},
	[91029] = {"Rath'thul Moonvale", {109693}, L["Profession Quest"]},
	[91404] = {"Samantha Scarlet", {109125,109124,109128,109127,109129,109126}, L["Profession Quest"]},
	[91031] = {"Nicholas Mitrik", {109125,109124,109128,109127,109129,109126}, L["Profession Quest"]},
	[90894] = {"Alexi Hackercam", {109118,109119}, L["Profession Quest"]},
	[91030] = {"Trixxy Volt", {109118,109119}, L["Profession Quest"]},
	--http://www.wowhead.com/guides/garrisons/town-hall-daily-quests
	--relic
	[89793] = {"Harrison Jones", {{{1500000,92601}}}, L["Relic Quest / Treasure Contract"]},
	[92223] = {"Surveyor Daltry", {{{1500000,92601}}}, L["Relic Quest / Treasure Contract"]},
}

GH.tradingpostIDs = {
	["buildingID"] = 145,
	["garrisonResourcesId"] = 824,
}
function GH:isDailyVisitorsEnabled()
	if type(GH.db.profile.tradingpostsupport)==nil then GH.db.profile.tradingpostsupport=true end
	return GH.db.profile.tradingpostsupport
end


function events.PLAYER_TARGET_CHANGED()
	GH:TradingPostTrader("target")
	GH:TownHallDailyVisitor("target")
end
function events.UPDATE_MOUSEOVER_UNIT()
	GH:TradingPostTrader("mouseover")
	GH:TownHallDailyVisitor("mouseover")
end
function GH:TradingPostTrader(t)
	if GH:isDailyVisitorsEnabled() then
		for id, arr in pairs(GH.npcMessages) do
			if GH:isNPC(id,t) then
				local timeout = 300 -- 300s (5min) between same print outs
				if not arr[4] or arr[4]+timeout < time() then
					local p = GH:CreateSaleString(id,arr)
					GH:print(L["On sale"]..": "..p,true);
					arr[4] = time();
					GH.db.profile.tradingpostnpc = {}
					GH.db.profile.tradingpostnpc.id = id
					GH.db.profile.tradingpostnpc.arr = arr
					GH.db.profile.tradingpostnpc.validuntil = time()+GetQuestResetTime()
				end
				break
			end
		end
	end
end

function GH:TownHallDailyVisitor(t)
	if GH:isDailyVisitorsEnabled() then
		for id, arr in pairs(GH.townhallNPCs) do
			if GH:isNPC(id,t) then
				local timeout = 300 -- 300s (5min) between same print outs
				if not arr[4] or arr[4]+timeout < time() then
					local p = GH:CreateDailyVisitorString(id,arr)
					GH:print(L["Daily Visitor"]..": "..p,true);
					arr[4] = time();
					-- only remember, if it is a profession vendor
					if arr[3]==L["Profession Quest"] then
						GH.db.profile.townhallnpc = {}
						GH.db.profile.townhallnpc.id = id
						GH.db.profile.townhallnpc.arr = arr
						GH.db.profile.townhallnpc.arr[3] = nil
						GH.db.profile.townhallnpc.validuntil = time()+GetQuestResetTime()
					end
				end
				break
			end
		end
	end
end

function GH:CreateDailyVisitorString(id,arr,iconsize)
	iconsize = iconsize or 24
	local space = ""
	if iconsize > 12 then space = " " end
	local p = ""
	local tt = {{left="possessions"..":"}}
	if arr[3] then
		p = p .. arr[3] .. " ("
	end
	GH.db.profile.tradingpostoption = GH.db.profile.tradingpostoption or "icons"
	local k = 1
	local l = 1
	local beginTag = ""
	local endTag = ""
	if GH.db.profile.tradingpostoption == "icons" then
		k = 10 -- 10:texture
		l = 10 -- 10:texture
		beginTag = "\124T"
		endTag = ":"..iconsize.."\124t"
	elseif GH.db.profile.tradingpostoption == "short" then
		k = 7 -- 6:class, 7:sub-class
		l = 1 -- 1: name
	elseif GH.db.profile.tradingpostoption == "links" then
		k = 1 -- 2:link
		l = 1 -- 2:link
	end
	for j=1,#arr[2] do
		
		if type(arr[2][j])=="number" then
			tt[#tt+1] = {left=beginTag .. select(k,GetItemInfo(arr[2][j]))  .. endTag, right=" x " .. GetItemCount(arr[2][j], true)}
			if GH.db.profile.tradingpostoption ~= "short" or j==1 then
				p = p .. space .. beginTag .. select(k,GetItemInfo(arr[2][j])) .. endTag .. space
			end
		else
			if j>1 then
				if #arr[2]==j then
					p = p .. " " .. L["or"] .. " " 
				else
					p = p .. ", " 
				end
			end
			for i=1,#arr[2][j] do
				if i>1 then
					p = p .. "& "
				end
				if arr[2][j][i][2]==92601 then
					p = p .. GetCoinTextureString(arr[2][j][i][1])
				else
					p = p .. arr[2][j][i][1] .. space .. beginTag .. select(l,GetItemInfo(arr[2][j][i][2])) .. endTag .. space
				end
			end
		end
	end

	if arr[3] then
		p =  strtrim(p) .. ")"
	end
	return p, tt
end
function GH:DailyVisitorsTimeReset()
	for id, arr in pairs(GH.npcMessages) do arr[4]=nil end
	for id, arr in pairs(GH.townhallNPCs) do arr[4]=nil end
end

function GH:CreateSaleString(id,arr,iconsize)
	iconsize = iconsize or 24
	local p = ""
	local tt = {{left="possessions"..":"}}
	if #arr[3]==0 then
		p = p .. " "..L["nothing"]
	else
		GH.db.profile.tradingpostoption = GH.db.profile.tradingpostoption or "icons"
		if GH.db.profile.tradingpostoption == "icons" then
			for i=1,#arr[3] do
				p = p .. "\124T" .. select(10,GetItemInfo(arr[3][i]))  ..":"..iconsize.."\124t "
				tt[#tt+1] = {left="\124T" .. select(10,GetItemInfo(arr[3][i]))  ..":"..iconsize.."\124t", right=" x " .. GetItemCount(arr[3][i], true)}
			end
		else
			p = p .. arr[2]
			for i=1,#arr[3] do
				tt[#tt+1] = {left=select(1,GetItemInfo(arr[3][i])), right=" x " .. GetItemCount(arr[3][i], true)}
			end
		end
	end
	if #tt==1 then
		tt=nil
	end
	return p,tt
end

-- register event listener
GarrisonHelper_Events:SetScript("OnEvent", function(self, event, ...) events[event](self, event, ...); end);
for event, func in pairs(events) do GarrisonHelper_Events:RegisterEvent(event); end

function GH:GetDailyVisitorsTooltip()
	if GH.db.profile.showstatic then
		local b = true
		local d = {}
		local buildingName = select(2,C_Garrison.GetBuildingInfo(GH.tradingpostIDs["buildingID"]))
		local tip = GH.tip
		if GH:isDailyVisitorsEnabled() and --[[ select(2,GetCurrencyInfo(GH.tradingpostIDs["garrisonResourcesId"]))>0 and ]]GH.db.profile.tradingpostnpc and GH.db.profile.tradingpostnpc.arr and GH.db.profile.tradingpostnpc.validuntil and GH.db.profile.tradingpostnpc.validuntil>=time() then
			local p,t = GH:CreateSaleString(GH.db.profile.tradingpostnpc.id,GH.db.profile.tradingpostnpc.arr,24)
			d[#d+1] = {left="|cffF5DA81"..tostring(buildingName), header=true}
			d[#d+1] = {left="   "..L["On sale"]..":"..(GH.db.profile.tradingpostoption == "icons" and "\n\n" or ""), right=p, tooltip=t}
			b=false
		end
		
		if GH:isDailyVisitorsEnabled() and GH.db.profile.tradingpostworkorder and GH.db.profile.tradingpostworkorder.itemID and GH.db.profile.tradingpostworkorder.itemName and GH.db.profile.tradingpostworkorder.validuntil and GH.db.profile.tradingpostworkorder.validuntil>=time() then
			if b then
				d[#d+1] = {left="|cffF5DA81"..tostring(buildingName), header=true}
			end
			local p,t = GH:CreateSaleString(nil,{nil,GH.db.profile.tradingpostworkorder.itemName,{GH.db.profile.tradingpostworkorder.itemID}},24)
			d[#d+1] = {left="   "..L["Work Orders for"]..":", right=p, tooltip=t}
		end
		
		if GH:isDailyVisitorsEnabled() and GH.db.profile.townhallnpc and GH.db.profile.townhallnpc.arr and GH.db.profile.townhallnpc.validuntil and GH.db.profile.townhallnpc.validuntil>=time() then
			local p,t = GH:CreateDailyVisitorString(GH.db.profile.townhallnpc.id,GH.db.profile.townhallnpc.arr,24)
			d[#d+1] = {left="|cffF5DA81"..tostring(GH.db.profile.localisedtownhall or "Town Hall"), header=true}
			d[#d+1] = {left="   "..L["Daily Trade Good"]..":"..(GH.db.profile.tradingpostoption == "icons" and "\n\n" or ""), right=p, tooltip=t}
		end
		
		if #d>0 then
			local tip = GH.tip
			GH:AddTooltipSeparator()
			for i=1,#d do
				if d[i].header then
					tip:AddHeader(d[i].left);
				else
					local lineIndex = tip:AddLine(d[i].left,  d[i].right);
					if d[i].tooltip then
						GH:AddMouseoverTooltip(tip, lineIndex, d[i].tooltip)
					end
				end
			end
		end
	end
end

--[[
	Needed, because if cache is empty, this information have to be retrieved from server.
]]
function GH:PreLoadDailyVisitorsItemInfos()
	for id, arr in pairs(GH.npcMessages) do
		for i=1,#arr[3] do
			GetItemInfo(arr[3][i])
		end
	end
	for id, arr in pairs(GH.townhallNPCs) do
		for j=1,#arr[2] do
			if type(arr[2][j])=="number" then
				GetItemInfo(arr[2][j])
			else
				for i=1,#arr[2][j] do
					GetItemInfo(arr[2][j][i][2])
				end
			end
		end
	end
end
GH:PreLoadDailyVisitorsItemInfos() -- TODO move this to an event like addon loaded